package com.pj.project.tb_person_black;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.global.BusinessException;
import com.pj.current.netty.command.constants.CommandType;
import com.pj.current.netty.command.handler.dto.BlackPersonDTO;
import com.pj.project.tb_person_filing.TbPersonFiling;
import com.pj.project.tb_terminal.TbTerminal;
import com.pj.project.tb_terminal.TbTerminalService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;

import javax.annotation.Resource;

/**
 * Service: tb_person_black -- 人员黑名单
 *
 * @author qzyReal
 */
@Service
@Transactional
@Slf4j
public class TbPersonBlackService extends ServiceImpl<TbPersonBlackMapper, TbPersonBlack> implements IService<TbPersonBlack> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbPersonBlackMapper tbPersonBlackMapper;

    @Resource
    @Lazy
    private TbTerminalService tbTerminalService;


    /**
     * 增
     */
    public void add(TbPersonBlack t) {
        if (!IdcardUtil.isValidCard(t.getIdCard())) {
            throw new BusinessException("身份证不正确");
        }
        TbPersonBlack db = findByIdCard(t.getIdCard());
        if (db != null) {
            throw new BusinessException("人员已经存在");
        }
        this.save(t);
        handlerIssued(t);
    }

    private void handlerIssued(TbPersonBlack t) {
        LocalDateTime startTime = t.getStartTime().toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
        LocalDateTime endTime = t.getEndTime().toInstant().atZone(ZoneId.systemDefault()).toLocalDateTime();
        LocalDateTime now = LocalDateTime.now();
        if (startTime.isBefore(now) && endTime.isAfter(now)) {
            issued(t);
        }
        if (now.isAfter(endTime)) {
            deleteTerminalBlack(t);
        }
    }

    /**
     * 删
     */
    public void delete(Long id) {
        TbPersonBlack db = this.getById(id);
        this.removeById(id);
        deleteTerminalBlack(db);
    }

    /**
     * 改
     */
    public void update(TbPersonBlack t) {
        if (!IdcardUtil.isValidCard(t.getIdCard())) {
            throw new BusinessException("身份证不正确");
        }
        TbPersonBlack idCardDb = findByIdCard(t.getIdCard());
        if (idCardDb != null && !idCardDb.getId().equals(t.getId())) {
            throw new BusinessException("人员已存在");
        }
        TbPersonBlack db = this.getById(t.getId());
        //改了身份证或者名字,部门，要把旧的人员信息清除，下发新的
        if (!StrUtil.equals(t.getIdCard(), db.getIdCard())
                || !StrUtil.equals(t.getName(), db.getName())
        ) {
            deleteTerminalBlack(db);
            issued(t);
        }
        handlerIssued(t);
        this.updateById(t);
    }

    /**
     * 删除设备黑名单
     *
     * @param t
     */
    public void deleteTerminalBlack(TbPersonBlack t) {
        List<TbTerminal> tbTerminalList = tbTerminalService.list();
        JSONObject params = new JSONObject();
        params.putIfAbsent("identity_numbers", Collections.singleton(t.getIdCard()));
        tbTerminalList.forEach(tbTerminal -> {
            tbTerminalService.setCommand(CommandType.DELETE_BLACKLIST_COMMAND.getFuncId(), params, tbTerminal);
            tbTerminalService.startHeartCommand(tbTerminal.getSn());
        });

    }


    /**
     * 查
     */
    public TbPersonBlack getById(Long id) {
        return tbPersonBlackMapper.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbPersonBlack> getList(SoMap so) {
        return tbPersonBlackMapper.getList(so);
    }

    /**
     * 下发所有黑名单到某一个设备
     *
     * @param t
     */
    public void issued(TbTerminal t) {
        List<TbPersonBlack> list =   findInBlackList(DateUtil.now());
        list.forEach(tbPersonBlack -> {
            JSONObject users = builderData(tbPersonBlack);
            tbTerminalService.setCommand(CommandType.ISSUED_BLACKLIST_COMMAND.getFuncId(), users, t);
        });
    }

    /**
     * 查找此时在黑名单的人员
     * @param now
     * @return
     */
    public List<TbPersonBlack> findInBlackList(String now) {
        QueryWrapper<TbPersonBlack>ew=new QueryWrapper<>();
        ew.ge("date_format(end_time,'%Y-%m-%d %H:%i:%s')", now)
                .le("date_format(start_time,'%Y-%m-%d %H:%i:%s')", now);
        return list(ew);
    }

    /**
     * 下发单个黑名单到部门所有设备
     *
     * @param tbPersonBlack
     */
    public void issued(TbPersonBlack tbPersonBlack) {
        List<TbTerminal> terminals = tbTerminalService.list();
        JSONObject users = builderData(tbPersonBlack);
        terminals.forEach(tbTerminal -> {
            log.error("下发黑名单,users:{},{}", tbTerminal.getSn(), JSONUtil.toJsonStr(users));
            tbTerminalService.setCommand(CommandType.ISSUED_BLACKLIST_COMMAND.getFuncId(), users, tbTerminal);
            tbTerminalService.startHeartCommand(tbTerminal.getSn());
        });
    }

    /**
     * 构建黑名单数据格式
     *
     * @param tbPersonBlack
     * @return
     */
    private JSONObject builderData(TbPersonBlack tbPersonBlack) {
        BlackPersonDTO dto = new BlackPersonDTO();
        dto.setIdentity_number(tbPersonBlack.getIdCard()).setName(tbPersonBlack.getName());
        JSONObject users = new JSONObject();
        users.putIfAbsent("users", dto);
        return users;
    }

    /**
     * 根据部门查找
     *
     * @param deptId
     * @return
     */
    private List<TbPersonBlack> findByDeptId(Long deptId) {
        QueryWrapper<TbPersonBlack> ew = new QueryWrapper<>();
        ew.eq("dept_id", deptId);
        return list(ew);
    }

    public TbPersonBlack findByIdCard(String idCard) {
        QueryWrapper<TbPersonBlack> ew = new QueryWrapper<>();
        ew.eq("id_card", idCard);
        return getOne(ew);
    }



    public TbPersonBlack findByIdCardAndTime(String idCard, String now) {
        QueryWrapper<TbPersonBlack> ew = new QueryWrapper<>();
        ew.lambda().eq(TbPersonBlack::getIdCard, idCard);
        ew.ge("date_format(end_time,'%Y-%m-%d %H:%i:%s')", now)
                .le("date_format(start_time,'%Y-%m-%d %H:%i:%s')", now);
        return getOne(ew);
    }

    public List<TbPersonBlack> findStartTimeEqTime(String time) {
        QueryWrapper<TbPersonBlack> ew = new QueryWrapper<>();
        ew.eq("date_format(start_time,'%Y-%m-%d %H:%i')", time);
        return list(ew);
    }

    public List<TbPersonBlack> findEndTimeEqTime(String time) {
        QueryWrapper<TbPersonBlack> ew = new QueryWrapper<>();
        ew.eq("date_format(end_time,'%Y-%m-%d %H:%i')", time);
        return list(ew);
    }
}
